// -*- C++ -*-
/*!
 * @file  SpeechSynthesizer.cpp
 * @brief Speech Synthesizer Component
 * @date $Date$
 *
 * $Id$
 */

#include <AquesTalk.h>

#include "SpeechSynthesizer.h"
#include "nkf.h"

#define BUF_SIZE 4096

typedef int (__stdcall *FUNC_Setting)(LPCSTR);
typedef void (__stdcall *FUNC_Convert1)(LPCTSTR, LPCTSTR);
typedef void (__stdcall *FUNC_Convert2)(LPCTSTR, DWORD, LPCTSTR, DWORD);

static HINSTANCE hd;

// Module specification
// <rtc-template block="module_spec">
static const char* speechsynthesizer_spec[] =
  {
    "implementation_id", "SpeechSynthesizer",
    "type_name",         "SpeechSynthesizer",
    "description",       "Speech Synthesizer Component",
    "version",           "1.0.0",
    "vendor",            "Kyusyu Institute of Technology",
    "category",          "kyutech",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
SpeechSynthesizer::SpeechSynthesizer(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_textIn("text", m_text),
    m_resultOut("result", m_result)

    // </rtc-template>
{
	if (hd == NULL) {
		hd = LoadLibrary("nkf32.dll");
	}
	if (hd == NULL) {
		printf("Fail to load nkf32 library.\n");
	}
}

/*!
 * @brief destructor
 */
SpeechSynthesizer::~SpeechSynthesizer()
{
}



RTC::ReturnCode_t SpeechSynthesizer::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("text", m_textIn);
  
  // Set OutPort buffer
  addOutPort("result", m_resultOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  std::cout << "Initialized: " << speechsynthesizer_spec[5] << "(" << speechsynthesizer_spec[7] << ")" << std::endl;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t SpeechSynthesizer::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SpeechSynthesizer::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SpeechSynthesizer::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SpeechSynthesizer::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SpeechSynthesizer::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t SpeechSynthesizer::onExecute(RTC::UniqueId ec_id)
{
	
	if (m_textIn.isNew()) {
		m_textIn.read();
		//std::cout << "Received: " << m_text.data << std::endl;
		//std::cout << "TimeStamp: " << m_text.tm.sec << "[s] ";
		//std::cout << m_text.tm.nsec << "[ns]" << std::endl;

		unsigned char *wav = NULL;

		// 
		if (m_text.data != NULL) {
			// ϊpobt@
			char buf[BUF_SIZE];
			// NKF̊֐
			FUNC_Setting SetNkfOption = (FUNC_Setting)GetProcAddress(hd,"SetNkfOption");
			FUNC_Convert1 NkfConvert = (FUNC_Convert1)GetProcAddress(hd,"NkfConvert");

			SetNkfOption("-s");
			NkfConvert(buf, m_text.data);

			// NkfConvertSafe̎gs
			//FUNC_Convert2 NkfConvertSafe = (FUNC_Convert2)GetProcAddress(hd,"NkfConvertSafe");
			//NkfConvertSafe(buf, BUF_SIZE - 1, sentence, strlen(sentence));

			int code = 0;
			wav = AquesTalk_Synthe(buf, 100, &code);
			if (code >= 102 && code <= 108 || code == 111
				|| code == 200 || code == 201 || code == 202 || code == 204) {
					std::cout << "AquesTalk_Synthe Error: code=" << code << std::endl;
					if (wav != NULL) {
						AquesTalk_FreeWave(wav);
						wav = NULL;
					}
			}
			else if (wav != NULL && code > 0) {
				int wav_size = code;
				std::cout << "AquesTalk_Synthe: len=" << code << ",text=" << buf << std::endl;
				m_result.data.length(wav_size);
				for (int i = 0; i < wav_size; i++) {
					m_result.data[i] = wav[i];
				}
				m_resultOut.write();
			}
		}

		// f[^̃NA
		if (wav != NULL) {
			AquesTalk_FreeWave(wav);
			//std::cout << "FreeWave" << std::endl;
			wav = NULL;
		}

	}
	//coil::usleep(1000);

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t SpeechSynthesizer::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SpeechSynthesizer::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SpeechSynthesizer::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SpeechSynthesizer::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SpeechSynthesizer::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void SpeechSynthesizerInit(RTC::Manager* manager)
  {
    coil::Properties profile(speechsynthesizer_spec);
    manager->registerFactory(profile,
                             RTC::Create<SpeechSynthesizer>,
                             RTC::Delete<SpeechSynthesizer>);
  }
  
};


